import json
import xml.etree.ElementTree as ET
import yaml
import re
from jsonschema import validate, ValidationError
from pathlib import Path


class ValidationReport:
    """
    A class to generate and print the final validation report.
    """
    def __init__(self, metadata_validator, license_checker, purl_validator):
        self.metadata_validator = metadata_validator
        self.license_checker = license_checker
        self.purl_validator = purl_validator

    def generate_report(self):
        """Generate a validation report with all errors and warnings."""
        print("\nValidation Report:")
        print("------------------")

        # Metadata Validation
        if self.metadata_validator.errors:
            print("Metadata Errors:")
            for error in self.metadata_validator.errors:
                print(f"  - {error}")
        else:
            print("Metadata validation passed.")

        # License Validation
        if self.license_checker.blocked_licenses:
            print("Blocked Licenses:")
            for entry in self.license_checker.blocked_licenses:
                component = entry["component"]
                matched_spdx_id = entry["matched_spdx_id"]
                print(
                    f"  - Component '{component.get('name')}' "
                    f"uses blocked license {matched_spdx_id}"
                )
        else:
            print("No blocked licenses detected.")

        # PURL Validation
        if self.purl_validator.problematic_packages:
            print("Problematic Packages:")
            for component, signature in (
                self.purl_validator.problematic_packages
            ):
                print(
                    f"  - Component '{component.get('name')}' "
                    f"with PURL '{component.get('purl')}'"
                )
                print(
                    f"    Matched Signature: "
                    f"{signature.get('package_name')} "
                    f"[{signature.get('problem_type')}]"
                )
        else:
            print("No problematic packages detected.")



import click
import os
from pathlib import Path
from ossbomer.sbom_parser import SBOMParser
from ossbomer.metadata_validator import MetadataValidator
from ossbomer.license_validator import LicenseValidator
from ossbomer.purl_validator import PurlValidator
from ossbomer.validation_report import ValidationReport
from ossbomer.dataset_manager import update_datasets, get_inventory
from ossbomer.dataset_manager import update_package_signatures


@click.group()
def main():
    """OSSBOMER CLI for validating SBOMs."""
    pass


@main.command()
@click.argument("sbom_path")
def validate(sbom_path):
    try:
        parser = SBOMParser(sbom_path)
    except ValueError as e:
        click.echo(f"Error: {e}")
        return
    package_dir = Path(__file__).resolve().parent
    dataset_dir = package_dir / "datasets"
    dataset_dir.mkdir(parents=True, exist_ok=True)

    # Metadata Validation
    metadata_validator = MetadataValidator(parser, dataset_dir)
    metadata_validator.validate_metadata()
    metadata_validator.validate_schema()
    metadata_validator.validate_components()

    # License Validation
    license_validator = LicenseValidator(parser, dataset_dir)
    license_validator.check_licenses()

    # PURL Validation
    purl_validator = PurlValidator(parser, dataset_dir, license_validator)
    purl_validator.check_purls()
    license_validator = purl_validator._license_validator()

    # Generate and Print Report
    report = ValidationReport(
        metadata_validator, license_validator, purl_validator
    )
    report.generate_report()


@main.command()
def update():
    update_datasets()
    update_package_signatures()
    click.echo("Datasets updated successfully!")


@main.command()
def version():
    from ossbomer import __version__
    click.echo(f"OSSBOMER version: {__version__}")


@main.command()
def inventory():
    get_inventory()



import json
import xml.etree.ElementTree as ET
import yaml
import re
from jsonschema import validate, ValidationError
from pathlib import Path


class SBOMParser:
    """
    A class to parse SBOM files in SPDX or CycloneDX formats.
    """
    def __init__(self, file_path):
        self.file_path = file_path
        self.data = None
        self.format = None
        self.load_sbom()

    def load_sbom(self):
        if self.file_path.endswith(('.json', '.yaml', '.yml')):
            self.data = self._load_json_or_yaml()
        elif self.file_path.endswith('.xml'):
            self.data = self._load_xml()
        else:
            raise ValueError(
                f"Unsupported file format. "
                f"Only JSON, YAML, and XML are supported."
            )
        self.format = self.detect_format()

    def _load_json_or_yaml(self):
        """Load a JSON or YAML SBOM file."""
        with open(self.file_path, 'r') as file:
            if self.file_path.endswith(('.yaml', '.yml')):
                return yaml.safe_load(file)
            return json.load(file)

    def _load_xml(self):
        """Load and parse an XML SBOM file."""
        tree = ET.parse(self.file_path)
        root = tree.getroot()
        return self._element_to_dict(root)

    def _element_to_dict(self, element):
        parsed_data = {element.tag: {}}
        for child in element:
            parsed_data[element.tag][child.tag] = self._element_to_dict(child)
        if element.attrib:
            parsed_data[element.tag]['@attributes'] = element.attrib
        if element.text and element.text.strip():
            parsed_data[element.tag]['#text'] = element.text.strip()
        return parsed_data

    def detect_format(self):
        if 'spdxVersion' in self.data:
            return 'spdx'
        elif (
            'bomFormat' in self.data and
            self.data['bomFormat'] == 'CycloneDX'
        ):
            return 'cyclonedx'
        else:
            raise ValueError(
                f"Unknown SBOM format. "
                f"Supported formats are SPDX and CycloneDX."
            )



import requests
import os
from pathlib import Path
import json
from tqdm import tqdm


def get_dataset_directory():
    """Get the directory where datasets are stored in the package."""
    package_dir = Path(__file__).resolve().parent
    dataset_dir = package_dir / "datasets"
    dataset_dir.mkdir(parents=True, exist_ok=True)
    return dataset_dir


def get_dataset_repository():
    base_url = "https://raw.githubusercontent.com/"
    gh_repo = "Xpertians/ossbomer-dataset"
    gh_branch = "/main"
    return base_url + gh_repo + gh_branch


def fetch_remote_file(url, local_path):
    response = requests.get(url, stream=True)
    if response.status_code != 200:
        raise Exception(f"Failed fetch {url}. status: {response.status_code}")

    total_size = int(response.headers.get('content-length', 0))
    if total_size == 0:
        raise Exception(f"The file at {url} is empty or unavailable.")

    with open(local_path, "wb") as file, tqdm(
        desc=f"Downloading {local_path.name}",
        total=total_size,
        unit="B",
        unit_scale=True,
        unit_divisor=1024,
    ) as bar:
        for data in response.iter_content(chunk_size=1024):
            file.write(data)
            bar.update(len(data))


def is_update_needed(local_path, remote_url):
    """Check if the local dataset needs to be updated by comparing versions."""
    if not local_path.exists():
        return True

    # Fetch the remote dataset
    response = requests.get(remote_url)
    if response.status_code != 200:
        raise Exception(
            f"Failed to fetch dataset from {remote_url}. "
            f"HTTP status: {response.status_code}"
        )

    remote_data = response.json()
    remote_version = remote_data.get("version")
    if not remote_version:
        raise ValueError("Remote dataset does not contain a 'version' field.")

    # Load the local dataset
    with open(local_path, "r") as file:
        local_data = json.load(file)
        local_version = local_data.get("version", "0.0.0")

    return remote_version > local_version


def update_datasets():
    dataset_dir = get_dataset_directory()
    dataset_repo = get_dataset_repository()

    datasets = {
        "license_rules.json": {
            "file_url": dataset_repo + "/license_rules.json"
        }
    }

    for file_name, urls in datasets.items():
        local_path = dataset_dir / file_name
        file_url = urls["file_url"]

        try:
            if is_update_needed(local_path, file_url):
                print(f"Updating {file_name}...")
                fetch_remote_file(file_url, local_path)
                print(f"{file_name} updated successfully!")
            else:
                print(f"{file_name} is already up-to-date.")
        except Exception as e:
            print(f"Error updating {file_name}: {e}")


def update_package_signatures():
    dataset_dir = get_dataset_directory() / "package_signatures"
    dataset_dir.mkdir(parents=True, exist_ok=True)
    dataset_repo = get_dataset_repository()
    remote_base_url = dataset_repo + "/package_signatures/"
    index_url = f"{remote_base_url}index.json"
    response = requests.get(index_url)
    if response.status_code != 200:
        raise Exception(
            f"Failed to fetch package signature index. "
            f"HTTP Status: {response.status_code}"
        )

    package_files = response.json().get("packages", [])
    updated_files = []

    for package_file in package_files:
        file_url = f"{remote_base_url}{package_file}"
        local_path = dataset_dir / package_file

        try:
            if is_update_needed(local_path, file_url):
                print(f"Updating {package_file}...")
                fetch_remote_file(file_url, local_path)
                updated_files.append(package_file)
                print(f"{package_file} updated successfully!")
            else:
                print(f"{package_file} is already up-to-date.")
        except Exception as e:
            print(f"Error updating {package_file}: {e}")

    # Regenerate the local index.json
    regenerate_local_index(dataset_dir)
    print("Local index.json regenerated successfully!")


def regenerate_local_index(dataset_dir):
    package_files = [f.name for f in dataset_dir.glob("*.json")]
    index_data = {"packages": package_files}

    index_path = dataset_dir / "index.json"
    with open(index_path, "w") as file:
        json.dump(index_data, file, indent=4)


def get_inventory():
    """Display an inventory of dataset files and their versions."""
    dataset_dir = get_dataset_directory()
    inventory = []
    dataset_repo = get_dataset_repository()

    # Add license rules to the inventory
    license_file = dataset_dir / "license_rules.json"
    license_remote_url = dataset_repo + "/license_rules.json"
    inventory.append(get_file_info(license_file, license_remote_url))

    # Add package signatures to the inventory
    package_dir = dataset_dir / "package_signatures"
    if package_dir.exists():
        for package_file in package_dir.glob("*.json"):
            if package_file.name == "index.json":
                continue  # Skip index.json
            remote_url = (
                f"{dataset_repo}/package_signatures/{package_file.name}"
            )
            inventory.append(get_file_info(package_file, remote_url))

    # Display the inventory
    print(
        f"{'File':<30} {'Local Version':<15} "
        f"{'Remote Version':<15} {'Status'}"
    )
    print("-" * 75)
    for item in inventory:
        print(
            f"{item['file']:<30} {item['local_version']:<15} "
            f"{item['remote_version']:<15} {item['status']}"
        )


def get_file_info(local_path, remote_url):
    """Retrieve information about a dataset file, including versions."""
    local_version = "N/A"
    remote_version = "N/A"
    status = "Not Downloaded"

    # Check local version
    if local_path.exists():
        try:
            with open(local_path, "r") as file:
                local_data = json.load(file)
                local_version = local_data.get("version", "Unknown")
        except Exception:
            local_version = "Error Reading"

    # Check remote version
    try:
        response = requests.get(remote_url)
        if response.status_code == 200:
            remote_data = response.json()
            remote_version = remote_data.get("version", "Unknown")
    except Exception:
        remote_version = "Error Fetching"

    # Determine status
    if local_version == "N/A":
        status = "Not Downloaded"
    elif local_version == remote_version:
        status = "Up-to-Date"
    elif remote_version != "Unknown" and local_version != remote_version:
        status = "Update Available"

    return {
        "file": local_path.name,
        "local_version": local_version,
        "remote_version": remote_version,
        "status": status,
    }



import json
import xml.etree.ElementTree as ET
import yaml
import re
from jsonschema import validate, ValidationError
from pathlib import Path


class PurlValidator:
    def __init__(self, sbom_parser, dataset_dir, license_validator):
        self.sbom_parser = sbom_parser
        self.schema_directory = Path(dataset_dir / "package_signatures")
        self.package_signatures_directory = self.schema_directory
        self.signatures = self._load_signatures()
        self._pkg_guidance = self.signatures
        self.problematic_packages = []
        self.license_validator = license_validator
        self.updated_blocked_licenses = []

    def _load_signatures(self):
        signatures = []
        for signature_file in self.package_signatures_directory.glob("*.json"):
            with open(signature_file, 'r') as file:
                signatures.append(json.load(file))
        return signatures

    def _license_validator(self):
        if self.license_validator.blocked_licenses:
            for entry in self.license_validator.blocked_licenses:
                component = entry["component"]
                pkg_purl = component.get('purl')
                if not self.check_purl(component, pkg_purl, False):
                    self.updated_blocked_licenses.append(entry)
        self.license_validator.blocked_licenses = self.updated_blocked_licenses
        return self.license_validator

    def check_purl(self, component, purl, blk):
        for signature in self.signatures:
            matched = False
            if "purls" in signature and purl in signature["purls"]:
                if signature["blocked"] and blk:
                    self.problematic_packages.append(
                        (component, signature)
                    )
                    matched = True
                if signature["approved"] and not blk:
                    return True

            if not matched and "regex" in signature:
                regex_list = signature["regex"]
                if isinstance(regex_list, str):
                    regex_list = [regex_list]
                for regex in regex_list:
                    if re.search(regex, purl):
                        if signature["blocked"] and blk:
                            self.problematic_packages.append(
                                (component, signature)
                            )
                        if signature["approved"] and not blk:
                            return True
                        break
        return False

    def check_purls(self):
        components = self.sbom_parser.data.get('components', [])
        for component in components:
            purl = component.get('purl')
            if not purl:
                print(
                    f"Warning: Missing PURL for component"
                    f" {component.get('name', 'unknown')}"
                )
                continue
            self.check_purl(component, purl, True)



from setuptools import setup, find_packages
import os

def get_version():
    version_file = os.path.join(os.path.dirname(__file__), "ossbomer", "__init__.py")
    with open(version_file, "r") as f:
        for line in f:
            if line.startswith("__version__"):
                delim = '"' if '"' in line else "'"
                return line.split(delim)[1]
    raise RuntimeError("Version not found in __init__.py")

with open("README", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="ossbomer",
    version=get_version(),
    description="SBOMs quality validator for Open Source License Compliance.",
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author="Oscar Valenzuela",
    author_email="oscar.valenzuela.b@gmail.com",
    license='Apache 2.0',
    url='https://github.com/Xpertians/xmonkey-ossbomer',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "click",
        "jsonschema",
        "requests",
        "PyYAML",
        "tqdm"
    ],
    package_data={
        "ossbomer": [
            "datasets/*.json",
            "datasets/package_signatures/*.json"
        ]
    },
    entry_points={
        "console_scripts": [
            "ossbomer=ossbomer.cli:main",
        ],
    },
    python_requires='>=3.6',
)



import json
from jsonschema import validate, ValidationError
from pathlib import Path
import re


class MetadataValidator:
    """
    A class to validate metadata, schema, and consistency for SBOM files.
    """
    def __init__(self, sbom_parser, schema_directory):
        self.sbom_parser = sbom_parser
        self.schema_directory = Path(schema_directory)
        self.errors = []

    def validate_metadata(self):
        data = self.sbom_parser.data
        missing_fields = []

        if self.sbom_parser.format == 'spdx':
            required_fields = [
                'spdxVersion', 'name', 'documentNamespace', 'creationInfo'
            ]
        elif self.sbom_parser.format == 'cyclonedx':
            required_fields = ['bomFormat', 'specVersion']
        else:
            raise ValueError("Unsupported SBOM format.")

        for field in required_fields:
            if field not in data:
                missing_fields.append(field)

        if missing_fields:
            self.errors.append(
                f"Missing metadata fields: {', '.join(missing_fields)}"
            )

    def validate_schema(self):
        schema_file = self._get_schema_file()
        with open(schema_file, 'r') as schema:
            schema_data = json.load(schema)
        try:
            validate(instance=self.sbom_parser.data, schema=schema_data)
        except ValidationError as e:
            self.errors.append(f"Schema validation error: {e.message}")

    def validate_components(self):
        components = self.sbom_parser.data.get('components', [])
        seen_ids = set()
        id_type = None
        expected_hash_set = None
        expected_hash_lengths = {
            "MD5": 32,
            "SHA-1": 40,
            "SHA-256": 64,
            "SHA-384": 96,
            "SHA-512": 128,
            "SHA3-256": 64,
            "SHA3-384": 96,
            "SHA3-512": 128
        }

        for component in components:
            # Validate unique IDs, it would need improvements
            comp_id = component.get('purl')
            if not comp_id:
                self.errors.append(f"Component missing ID: {component}")
            elif comp_id in seen_ids:
                self.errors.append(f"Duplicate component ID found: {comp_id}")
            else:
                seen_ids.add(comp_id)

            current_id_type = (
                "purl" if comp_id and comp_id.startswith("pkg:")
                else "other"
            )
            if id_type is None:
                id_type = current_id_type
            elif id_type != current_id_type:
                self.errors.append(f"Inconsistent ID type: {comp_id}")

            # Validate hash types and lengths
            hashes = component.get('hashes', [])
            hash_set = set()

            for hash_obj in hashes:
                hash_algo = hash_obj.get("alg")
                hash_value = hash_obj.get("content")
                if not hash_algo or not hash_value:
                    self.errors.append(
                        f"Invalid hash: Missing 'alg' "
                        f"or 'content' for {comp_id}"
                    )
                    continue
                hash_set.add(hash_algo)
                expected_length = expected_hash_lengths.get(hash_algo)
                if expected_length is None:
                    self.errors.append(
                        f"Unknown hash algorithm: {hash_algo} for {comp_id}"
                    )
                elif len(hash_value) != expected_length:
                    self.errors.append(
                        f"Invalid hash length for alg: {hash_algo}. "
                        f"Expected {expected_length}, "
                        f"got {len(hash_value)} for {comp_id}"
                    )
            if expected_hash_set is None:
                expected_hash_set = hash_set
            elif hash_set != expected_hash_set:
                self.errors.append(
                    f"Inconsistent hash algorithms in component {comp_id}. "
                    f"Expected: {expected_hash_set}, Found: {hash_set}"
                )

    def _get_schema_file(self):
        if self.sbom_parser.format == 'spdx':
            return self.schema_directory / "spdx_schema.json"
        elif self.sbom_parser.format == 'cyclonedx':
            return self.schema_directory / "cyclonedx_schema.json"
        else:
            raise ValueError("Unsupported SBOM format.")



import json
import xml.etree.ElementTree as ET
import yaml
import re
from jsonschema import validate, ValidationError
from pathlib import Path


class LicenseValidator:
    """
    A class to check licenses in the SBOM.
    """
    def __init__(self, sbom_parser, dataset_dir):
        self.sbom_parser = sbom_parser
        license_rules_file = Path(dataset_dir) / "license_rules.json"
        with open(license_rules_file, 'r') as file:
            self.license_rules = json.load(file)["licenses"]
        self.blocked_licenses = []

    def check_licenses(self):
        """Check for blocked licenses in the SBOM components."""
        components = self.sbom_parser.data.get('components', [])
        for component in components:
            license_declared = component.get('licenseDeclared', 'NOASSERTION')
            if license_declared in [
                rule["spdx_id"] for rule in self.license_rules
                if not rule["distribution"]
            ]:
                self.blocked_licenses.append(component)

            licenses = component.get("licenses", [])
            if not licenses:
                component_name = component.get('name', 'unknown')
                print(f"Warning: Missing license for '{component_name}'")
                continue

            for license_entry in licenses:
                license_id = license_entry.get("license", {}).get(
                    "id", "NOASSERTION"
                )
                if license_id.lower() == "noassertion":
                    component_name = component.get('name', 'unknown')
                    print(
                        f"Warning: 'noassertion' for '{component_name}'"
                    )
                    continue

                matching_rule = next(
                    (
                        rule for rule in self.license_rules
                        if rule["spdx_id"] == license_id
                    ),
                    None
                )
                if matching_rule:
                    if not matching_rule["distribution"]:
                        self.blocked_licenses.append({
                            "component": component,
                            "matched_spdx_id": license_id
                        })
                else:
                    print(
                        f"Warning: Unknown license '{license_id}' "
                        f"for '{component['name']}'"
                    )
